{
  cbview.pas
    v1.0 of 01/08/1999
      Component that registers itself as a clipboard viewer to egt notifications
      of when clipboard contents change.
}

{
  History
  - v1.0 of 01/08/1999
      Original version
}

{
  Dependencies
      NONE
}

{
  Legal notice
    This component is copyright  P.D.Johnson, 1999.
    The source code and help files can be freely distributed on a not-for-profit
    basis providing that:
      + the source code is not altered.
      + this readme file is distributed with it unchanged
    By not-for-profit I mean that you may recover out of pocket expenses
    incurred in distributing the code, but should not make a profit from this.
    If you discover any bugs in this implementation, or if you have any update
    suggestions, please contact me on peter.johnson@openlink.org.
    Please do modify the code for you own use. I'd like to see any changes you
    make - I could incorporate them into future versions. Please notify me of
    changes on at the above e-mail address.
    This software is provided as is - no warranty is given as to its suitability
    for any purposes to which you may wish to put it.
}


unit CBView;

interface

uses
  {$IFDEF WIN32}
    Windows,
  {$ELSE}
    WinProcs, WinTypes,
  {$ENDIF}
  Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TPJCBViewer = class(TComponent)
  private {properties}
    FOnClipboardChanged : TNotifyEvent;
    FTriggerOnCreation : Boolean;
    FEnabled : Boolean;
  private
    FHWnd : HWND;
      {Handle of hidden window used as a clipboard viewer}
    FHWndNextViewer : HWND;
      {Handle of next clipboard viewer window in chain}
    procedure ClipboardChanged;
      {Triggers OnClipboardChanged event if any handler is assigned and
      component is enabled}
  protected
    procedure Loaded; override;
      {Overrride of Loaded method to fire OnClipboardChanged event if required
      on component creation}
    procedure WndMethod(var Msg: TMessage); virtual;
      {Window procedure for hidden window}
  public
    constructor Create(AOwner : TComponent); override;
      {Class constructor - creates a hidden window and registers it as a
      clipboard viewer and sets default property values}
    destructor Destroy; override;
      {Class destructor - de-registers hidden window and frees it}
  published
    property OnClipboardChanged : TNotifyEvent
      read FOnClipboardChanged write FOnClipboardChanged;
      {Event triggered whenever clipboard contents change}
    property TriggerOnCreation : Boolean
      read FTriggerOnCreation write FTriggerOnCreation default True;
      {When true causes OnClipboardChanged event to be triggered as soon as
      component is created, otherwise OnClipboardChanged event only triggers
      when clipboard actually changes}
    property Enabled : Boolean read FEnabled write FEnabled default True;
      {When true component triggers events when clipboard changes, when false
      these events are not triggered}
  end;

procedure Register;

implementation

constructor TPJCBViewer.Create(AOwner : TComponent);
  {Class constructor - creates a hidden window and registers it as a clipboard
  viewer and sets default property values}
begin
  inherited Create(AOwner);
  {Create hidden window to be used as clipboard viewer}
  FHWnd := AllocateHWnd(WndMethod);
  {Register hidden window as a clipboard viewer and store handle of next window
  in chain}
  FHWndNextViewer := SetClipboardViewer(FHWnd);
  {Set default property values}
  FTriggerOnCreation := True;
  FEnabled := True;
end;

destructor TPJCBViewer.Destroy;
  {Class destructor - de-registers hidden window and frees it}
begin
  {Remove our hidden window from the chain of clipboard viewers}
  ChangeClipboardChain(FHWnd, FHWndNextViewer);
  {Get rid of the window}
  DeallocateHWnd(FHWnd);
  inherited Destroy;
end;

procedure TPJCBViewer.Loaded;
  {Overrride of Loaded method to fire OnClipboardChanged event if required on
  component creation}
begin
  inherited Loaded;
  {If we're notifying a clipboard change event on creation then do it}
  if FTriggerOnCreation then
    ClipboardChanged;
end;

procedure TPJCBViewer.WndMethod(var Msg : TMessage);
  {Window procedure for hidden window}
var
  Handled : Boolean;              {Flag true if we handle a message}
begin
	{Assume we've handled message}
  Handled := True;
  {Now process them, handling the ones we want}
  with Msg do
  begin
  	case Msg of
      WM_DRAWCLIPBOARD:
      begin
        {Trigger OnClipboardChanged event}
        ClipboardChanged;
        {Pass on message to next window in viewer chain (if any)}
        if FHWndNextViewer <> 0 then
          SendMessage(FHWndNextViewer, Msg, WParam, LParam);
      end;
      WM_CHANGECBCHAIN:
      begin
        {Check if handle of window being detached from chain is the next one to
        us}
        if HWND(WParam) = FHWndNextViewer then
          {Window being detached is next one - so record handle of new "next"
          window in chain}
          FHWndNextViewer := HWND(LParam)
        else
          {Window being detached is not next one so pass on message to next
          window if any)}
          if FHWndNextViewer <> 0 then
            SendMessage(FHWndNextViewer, Msg, WParam, LParam);
      end;
      else
      begin
        {We're not handling this message - record fact}
        Handled := False;
      end;
    end;
    {Check if we handled message}
    if Handled then
      {We did - return 0}
      Result := 0
    else
      {We didn't - pass message on to Windows}
      Result := DefWindowProc(FHWnd, Msg, WParam, LParam);
  end;
end;


procedure TPJCBViewer.ClipboardChanged;
  {Triggers OnClipboardChanged event if any handler is assigned and component is
  enabled}
begin
  try
    {Fire event if assigned and enabled}
    if Assigned(FOnClipboardChanged) and FEnabled then
      FOnClipboardChanged(Self);
  except
    {Ensure any un-caught exception is handled by application and then continue}
    Application.HandleException(Self);
  end;
end;

procedure Register;
begin
  RegisterComponents('PJ Stuff', [TPJCBViewer]);
end;

end.
